# Création de l'API {-}

## Idée générale {-}
Cette (+API_a) doit permettre au développeur de créer des stratégies de trading. L'objectif est de viser la simplicité en utilisant des graphes au travers desquels les données transitent. 
L'utilisation de graphes simplifie le développement car cela permet de transposer aisément une idée faite sur papier en code.

\cimg{figs/soft/exemple-trader-basic.png}{scale=0.5}{Exemple d'un algorithme de trading aléatoire avec Akka Graph}{Source : Réalisé par Paschoud Nicolas}

Cet exemple représente une stratégie de trading aléatoire. Les données boursière transites jusqu'à la sortie, `Sink[Transaction]`, en passant par des fonctions. Ces données, en passant par ces fonctions, sont modifiées et permettront de décider si il faut acheter ou vendre.

Les sources de données sont mises à disposition par le cluster Kafka et l'API offre un accès facilité à ces données. Elle offre aussi un accès au cluster afin de publier les données dans un topic pour y publier les données statistiques d'un acteur.

Cette manière de traiter les données ne sera pas seulement utile pour les traders, mais aussi pour développer des indicateurs techniques comme les (+MA_a), (+RSI_a), etc. L'API offre déjà quelques indicateurs mais aussi une interface sur laquelle se baser afin de créer son propre indicateur spécifique à la stratégie développée.

\newpage

## Configuration des librairies Akka {-}
L'ensemble des libriaires mises à disposition par Akka doivent être configurées via le fichier `application.conf` qui se trouve dans le dossier `resources` de la hiérarchie du projet sbt. Ce fichier contient d'importantes informations qui permettent aux outils Akka de fonctionner correctement et selon les désirs de l'utilisateur.

Un fichier de configuration de base est présent au sein de chaque projet utilisant les librairie Akka. Il est cependant possible de le personnaliser avec ses propres paramètres.
Les paramètres sont organisés par librairie au format (+HOCON_a).

```shell
akka {
    loglevel = "OFF"
    actor{
        ...
    }

    stream{
        ...
    }

    cluster{
        sharding{
            ...
        }
        ...
    }
}
```

Pour faciliter la vie du développeur, la récupération de la configuration se fait grâce à une classe statique. Cette classe statique met à disposition des fonctions permettant de récupérer des pramètres spécifiques comme ceux des consumer et producer Kafka.

\cimg{figs/soft/soft-config.png}{scale=0.5}{Schéma UML de la classe statique KafkaSettings}{Source : Réalisé par Paschoud Nicolas}

Cette classe est utilisée par la suite afin de mettre à disposition du développeur un accès à Kafka.

\newpage

## Les types de données {-}
Comme vu dans le chapitre sur la récupération et le stockage des données, seuls deux types de données sont disponibles.
Le développeur créant sa stratégie de trading a à sa disposition deux types d'actifs présents dans la base de données : `OHLC` et `AskBid`. Ce sont les deux types pris en entrée d'une stratégie.

\cimg{figs/soft/soft-type.png}{scale=0.5}{Schéma UML des types de données de l'API}{Source : Réalisé par Paschoud Nicolas}

Grâce à l'interface `DataType`, il est possible d'ajouter de nouveaux type de données selon les besoins de l'utilisateur. Il peut ainsi personnaliser ses stratégies en ajoutant de nouvelles façons de représenter les données. En revanche, les seules données brutes proposées par le système actuel sont `OHLC` et `AskBid`. Il devra donc appliquer des transformations afin de faire correspondre les données offertes à son format.

L'interface sera utile de la même manière qu'expliqué précédemment lorsque de nouveaux type de données brutes seront ajoutés à la base de données.

\newpage

## Accès au cluster Kafka à l'aide d'Alpakka {-}

### Consommer les données {-}
Pour faciliter la consommation des données boursières provenant du cluster Kafka, la classe statique `KafkaConsummer` offre via son interface, des sources de données prêtes à être utilisées. Elle permet de récupérer une source de `DataType` (le type de données vu précédement). Ces sources vont récupérer les données d'actifs qui se trouvent sur le cluster Kafka.

\cimg{figs/soft/soft-consume-topic.png}{scale=0.5}{Schéma UML de la classe statique KafkaConsummer retournant des sources de données boursières}{Source : Réalisé par Paschoud Nicolas}

Cinq fonctions permettent de récupérer des sources de données différentes.

\begin{itemize}
\item Crypto-monnaies
\item Forex
\item Stock
\item Order Book d'un actif
\item Statistiques des acteurs
\end{itemize}

Toutes ces fonctions publiques font appel à des fonctions privées permettant de se débarasser de la lourde tâche de récupération de la configuration Akka. Elles font exactement la même chose à la seule différence qu'elles catégorisent et nomment les consumer afin de les différencier. Chaque consumer Kafka doit avoir un nom et un groupe sans lesquels il ne peut pas consommer les données. Ils sont tous nommés de cette manière :
\begin{center}
Nom du client : consumerName-consumer-topicName
\end{center}
\begin{center}
Nom du groupe : consumerName-consumers
\end{center}

Deux paramètres sont nécessaires afin d'obtenir une source de données : le nom du topic et la méthode de récupération. Les noms des topics sont présentés au chapitre [Organisation des topics pour la sauvegarde des données boursières](#organisation-des-topics-pour-la-sauvegarde-des-données-boursières). La méthode de récupération premet de savoir si l'ensemble de l'historique de données doit être récupéré ou non.

\cimg{figs/soft/soft-historique-type.png}{scale=0.75}{Schéma UML représentant le type de consommation d'une source de données}{Source : Schéma réalisé par Paschoud Nicolas}

Si le paramètre `consumerType` vaut `Early`, la source de données contiendra l'historique de données complet de l'actif. A l'inverse, si il prend la valeur `Latest`, seul les nouvelles données seront consommées par la source.

Cette approche permet de récupérer en une ligne de code une `Source` de données d'un actif. Cet exemple montre comment récupérer les données OHLC de l'actif BTCEUR minute par minute.
```scala
val topic: String = "BTCEUR_m"
// Récupération de la source à l'aide de la classe statique KafkaConsumer
val srcBTCEUR: Source[OHLC, NotUsed] = KafkaConsumer.cryptoConsumer(topic, Latest)
srcBTCEUR.runWith(Sink.foreach(println))
```
Dans cet exemple, la source de données récupère les dernières informations boursières de l'actif **BTCEUR** minute par minute grâce au paramètre `Latest`. La source va ensuite être exécutée avec un `Sink` affichant chaque donnée.

\newpage

### Produire des données {-}
La production des données fonctionne de la même manière que la production de données. Une classe statique `KafkaProducer` offre via son interface une méthode qui retourne un `Sink` pour produire des données.

\cimg{figs/soft/soft-kafka-producer.png}{scale=0.75}{Schéma UML de la classe statique KafkaProducer}{Source : Réalisé par Paschoud Nicolas}

La méthode statique `produce` prenant en paramètre le nom du topic dans lequel les données vont être produites, va retourner un `Sink[String, NotUsed]`. Ce sink est configuré de sorte à ce qu'il se connecte à Kafka et envoie un message au topic lorsqu'une donnée est reçue.

```scala
val topic: String = "topicTest"
val src: Source[String, NotUsed] = Source(List(
    "Hello",
    "World",
    "!"
))
// Récupération du puit à l'aide de la classe statique KafkaProducer
val sink: Sink[String, NotUsed] = KafkaProducer.produce(topic)
src.runWith(sink)
```

Ici les données `Hello`, `World` et `!` seront produites dans le topic nommé `topicTest`. Les données produites doivent être sous forme de `String`. Elles sont produites dans l'ordre d'arrivée dans le `Sink`, une fois envoyée à Kafka, l'ordre d'écriture est immuable.

\newpage

## Développer un indicateur technique {-}
L'utilisateur de l'API a accès à différents indicateurs techniques ainsi qu'une interface lui permettant d'en développer lui même. Ces indicateurs prennent des `DataType` en entrée et retourne n'importe quel type de données. L'indicateur peut être un `Flow` ou une `Source`, l'interface propose ces deux implémentations.

\cimg{figs/soft/soft-indicator.png}{scale=0.6}{Schéma UML de l'interface Indicator}{Source : Réalisé par Paschoud Nicolas}

Cette interface permet à un développeur de créer ses propres indicateurs techniques. Certains indicateurs communs sont offerts par la librairie.

\begin{itemize}
\setlength\itemsep{-0.6em}
\item Moving Average ou MA
\item Exponential Moving Average
\item Weighted Moving Average
\item MACD
\item Bollinger
\item OBV
\item Stochastic
\end{itemize}

Le développement de ces indicateurs se fait à l'aide de graph Akka représenté en tant que `Flow` ou de source `Source`.

Cet exemple montre comment l'indicateur (+MACD_a) a été développé en tant que `Flow`. Le calcul de cette (+MACD_a) est basé sur la définition du site investopedia.com [@hayes_moving_2019].

```scala
def asFlow(): Flow[OHLC, Double, NotUsed] = Flow.fromGraph(GraphDSL.create() {
    implicit b => import GraphDSL.Implicits._
        val in = b.add(Broadcast[OHLC](2))
        val zip = b.add(ZipWith[Double, Double, Double]{
            case (ema12, ema26) => ema12 - ema26
        })
        in ~> EMA.asFlow(12) ~> zip.in0
        in ~> EMA.asFlow(26) ~> zip.in1
        FlowShape(in.in, zip.out)
    })
```

Cet indicateur utilise un autre indicateur développé précédemment, la (+EMA_a). Le calcul de la (+MACD_a) est assez simple, il s'agit de la différence entre une EMA de 12 périodes et d'une EMA de 26 périodes. Les données entrent par la fonction `in` qui est du type `Broadcast` et permet de distribuer les données au graph. Ces données vont ensuite passer à travers les deux EMA, `EMA.asFlow(12)` et `EMA.asFlow(26)` puis être transférées à la fonction `zip`. La fonction `zip` a pour objectif de rassembler les données des deux EMA pour pouvoir faire la soustraction nécessaire à l'obtention d'une MACD. Elle est la fonction de sortie du graph et c'est pour cette raison que les valeurs contenue dans la fonction `.out` sont passé en second paramètre de l'objet `FlowShape(in.in, zip.out)`. FlowShape a pour objectif de transformer le graph précédement construit en `Flow` au travers duquel n'importe quelles données du type `OHLC` vont être transfromées en `Double`.

## Comment créer un trader {-}

### Les messages d'un acteur {-}
Les acteurs communiquent les uns avec les autres par messages. Un acteur est définit par les messages qu'il envoie et reçoit.

\cimg{figs/soft/soft-trader-messages.png}{scale=0.5}{Schéma UML de l'interface et des messages d'un acteur de type Trader}{Source : Réalisé par Paschoud Nicolas}

Ce sont les messages de base que n'importe quel trader utilise pour communiquer. `Start` et `Stop` permettent, comme leur nom l'indique, de démarrer ou arrêter un acteur. `BuyAll` permet d'effectuer une transaction sur le marché pour la totalité de la valeur du portefeuille qu'il possède. A l'inverse, `SellAll` permet de vendre l'ensemble des positions actuelles. Ces deux derniers messages peuvent être envoyés par l'utilisateur lorsque les marchés s'emballent et que le comportement de la stratégie est trop hasardeuse.

### L'interface Trader {-}
La partie la plus importante de l'API est la création d'acteurs. Afin de faciliter la création de ceux-ci, une interface nommée `Trader` offre à l'utilisateur toute les méthodes nécessaire au bon fonctionnement d'un acteur. Dans cette interface, la plupart des comportements de base sont déjà développées, et l'utilisateur n'a qu'une seule fonction à implémenter.

\cimg{figs/soft/soft-trader.png}{scale=0.5}{Schéma UML de l'interface Trader}{Source : Réalisé par Paschoud Nicolas}

La seule fonction que l'utilisateur a à développer est la méthode `strategy`. Cette méthode est exécutée par la méthode `run` déjà implémentée.

#### La réception de messages {-}
Les comportements lors de la réception d'un message sont décrits dans la méthode `behave`. Il est possible de changer le comportement de cette méthode mais il est cependant conseillé de bien maîtriser la librairie Akka Actor avant de toucher quoi que ce soit. Lorsque l'acteur reçoit un message de type `Start`, la stratégie va être exécutée en utilisant la source de données passée en paramètre de la méthode `run`.

```scala
Behaviors.receiveMessage {
    case s: Start => {
        // Exécution de la stratégie
        source.via(strategy).via(statFlow).runWith(statSink)
        currentState = Running
        Behaviors.same
    }
}
```

Voici le comportement de l'acteur lors de la réception du message `Start`. Afin d'exécuter la stratégie, un stream est créé et exécuté à l'aide de la source de données et d'un sink de publications de statistiques dans Kafka. Avant d'atteindre le sink, les données vont passer par la stratégie `.via(strategy)` puis par un flow transformant les données de sortie de la stratégie en données statistiques.

#### La stratégie {-}
La méthode `strategy` est l'unique méthode qui n'est pas définie dans cette interface. C'est dans cette méthode que la stratégie de trading va être développée. La signature de cette méthode oblige l'utilisateur a créer un graph qui sera transformé en `Flow[T, Transaction, NotUsed]`.
Le type `T` dont le flow est contitué est un `DataType`. Cela veut dire que n'importe quelle classe héritant du type `DataType` peut être utilisé pour développer une stratégie. Voici la définition de l'interface (ou trait en Scala).

```scala
trait Trader [T <: DataType]
```

Elle est bornée au type `DataType` ce qui veut dire qu'un trader maîtrise un type de donnée particulier comme `OHLC` et `AskBid` (voir chapitre [Les types de données](#les-types-de-données)). Lors de l'implémentation d'un nouvelle classe de type `Trader`, l'utilisateur devra préciser le type de données que la stratégie maîtrisera.

De manière globale, la logique derrière chaque méthode peut être modifiée, celle de base permet en revanche de faciliter la vie de l'utilisateur en lui offrant le comportement minimum permettant l'exécution d'un acteur. Il est en revanche fortement conseillé de maîtriser les outils Akka et notamment Akka Actors avant de faire une quelconque modification.

## Mise en place du cluster Akka {-}
L'API permet de déployer et exécuter les acteurs directement sur un cluster. Il est possible de le déployer sur plusieurs machines. Ces machines acceuillent les acteurs déployés par le client et se charge de délivrer les messages qui leurs sont dédiés.

Le déploiement du cluster se fait à l'aide d'un programme développé dans l'API. Ce programme doit être configuré à l'aide des adresses IP de chacune des machines du cluster.

```shell
cluster {
    shutdown-after-unsuccessful-join-seed-nodes = 60s

    seed-nodes = [
      "akka://Distribia@ip:port",
      "akka://Distribia@ip:port"
    ]
  }
```

Ces adresses vont permettrent à n'importe quel acteur de se connecter au cluster lors du processus d'exécution de la stratégie. Un fois configurer, le programme doit être exécuté sur chacun des noeuds.


## Synthèse {-}
L'API développée permet de travailler assez simplement sans grande connaissance des outils utilisés. Elle est modulable et générique ce qui permet une personnalisation de l'outil au désir de l'utilisateur.

Les graphs Akka ont permis de développer une API simple et permet de développer des stratégies rapidement sans difficulter

\pagebreak