# Utiliser l'API {-}

## Créer une stratégie de trading {-}
Afin de développer une stratégie de trading, l'utilisateur doit créer une classe héritant de l'interface `Trader` décrite précédemment.
Avant toute chose, l'interface ou trait en Scala, Trader est typé et borné.

```scala
trait Trader [T <: DataType]
```

Ici un exemple de création de la classe `TakeProfitStopLoss` qui implémentera une stratégie se basant sur des supports et résistances permettant d'éviter de grosses pertes tout en faisant du bénéfice lorsque l'occasion se présente.

```scala
class RandomTrader (
            val asset: String,
            val invested: Double,
            val name: String,
        ) extends Trader [OHLC] {
```

Certaines valeurs  n'ayant pas été définies dans la classe `Trader`, elles devront être définies dans l'implémentation. Ces valeurs sont `traderName`, `asset` et `startingMoney`. Celles-ci sont définies dans le constructeur.

La seule méthode non définie étant `strategy`, l'utilisateur n'a que peu de travail d'implémentation à effectuer. La stratégie développée ici est très basique et ne possède aucune intelligence.
Il s'agit d'une stratégie de trading aléatoire qui permet de placer des ordres aléatoirement. Pour chaque entrée OHLC, une transactions sera effectuée que ce soit un achat ou une vente.

\newpage


```scala
    def strategy(): Flow[OHLC, Transaction, NotUsed] = Flow.fromGraph(GraphDSL.create() {
        implicit b =>
            import GraphDSL.Implicits._

            val in = b.add(Broadcast[OHLC](1))
            val g = in ~> decide

            FlowShape(in.in, g.outlet)
        }
    )
```

Cette implémentation de la méthode `strategy` correspond donc à la stratégie décrite précédement. La fonction `decide` en complément de `strategy` permet de tester les conditions d'achat ou de vente. Elle permet d'effectuer une transaction ou non à l'aide d'un nombre aléatoire. Cette fonction prend un `OHLC` en entrée et permet à la méthode d'accéder aux données qu'elle souhaite.

```scala
def decide(): Flow[OHLC, Transaction, NotUsed] = Flow[OHLC].map { ohlc => {
    if (r.nextDouble() > 0.5) {
        buy(ohlc.close, volume)
    } else {
        sell(ohlc.close, volume)
    }
}}
```

Cette fonction va aléatoirement placer un ordre pour chacune des données OHLC entrante dans le graph. C'est un comportement basique sans aucune intelligence.

Grâce à Akka Graph, il est très facile de créer un algorithme de trading. L'algorithme étant exécuté par les fonctions déjà présentes dans la librairie, l'utilisateur peut se concentrer sur sa stratégie et la développer très rapidement.

## Exécuter un trader {-}
Une fois une stratégie développée, il faut pouvoir l'exécuter afin de valider ses performances. Grâce à la librairie, il n'y a aucun travail à faire, les fonctions d'exécution étant prêtes. La stratégie étant développée dans une classe, elle doit être instanciée afin de pouvoir l'exécuter. La stratégie décrite précédemment sera utilisée pour la démonstration. Ce trader va travailler sur l'historique entier de l'actif BTCUSD minute par minute.

```scala
val topic: String = "BTCUSD_m"
val src: Source[OHLC, NotUsed] = KafkaConsumer.cryptoConsumer(topic, Early)
val trader: RandomTrader = new RandomTrader(topic, 10000, "Random")
val ref = trader.run(src)
```

Le trader est instancié avec des paramètres tel que le montant à investir, le nom de l'actif ainsi que le nom de l'agent qui pourra être utilisé pour accéder à ses statistiques.
Une fois la stratégie instanciée et configurée, elle est exécutée à l'aide de la méthode `run` décrite dans l'interface `Trader`. Cette méthode a besoin d'une source de données pour être exécutée. Elle est récupérée de la même manière qu'avant, à l'aide de `KafkaConsumer`.

La méthode `run` permet de déployer l'agent sur le cluster sans avoir à programmer quoi que ce soit. C'est l'un des avantage de l'API, le déploiement et l'exécution est automatique et il n'y a aucun travail à faire de se coter là pour l'utilisateur.

## Pile ou face {-}
Pour pouvoir tester et valider l'API, une petite stratégie de trading a été développée. Elle a permis de tester l'ensemble du système. Un ensemble contenant 500 données prise sur le cours du Bitcoin face au Dollar a été mis à disposition pour l'occasion.

\cimg{figs/data/extrait-BTCUSD.png}{scale=0.25}{Extrait du cours BTCEUR en date du dimanche 9 Août 2020 entre 10h28 et 18h51. Prix de fermeture}{Source : Données de la plateforme de trading kraken.com. Graphique réalisé par Paschoud Nicolas}

La stratégie la plus simple que l'on puisse imaginer, est bien évidemment une stratégie d'achat et de vente aléatoire. Voici les résultats d'une exécution sur le cours décrit précédemment.

\cimg{figs/data/random-profit-fake500.png}{scale=0.25}{Performance d'une stratégie de trading aléatoire}{Source : Résalisé par Paschoud Nicolas}

Ne possédant aucune logique ni intelligence, cette stratégie n'est pas très efficace. Pour chaque chandelier, l'algorithme a une chance sur deux d'acheter ou de vendre. L'investissement de départ étant de 15000 dollars, la stratégie a terminer sa course à 13800 dollars, ce traduisant par une perte de près de 1200 dollars.

## Gestion des agents présent sur le cluster {-}
Ce chapitre présente les outils développés permettant de gérer et visualiser les données des agents en cours d'exécution sur le cluster.

### Publication des statistiques des agents {-}
Chaque agent en cours d'exécution publie à chaque transaction un ensemble d'informations dans un topic nommé `statistics`. Ce topic enregistre toutes les statistiques de tous les agents qui sont et ont été exécutés. L'ensemble des outils suivants se basent sur ce topic afin de présenter les informations à l'utilisateur.

\newpage

### Gérer les agents à l'aide d'une console {-}
Une console est mise à disposition du développeur qui souhaite accéder au cluster afin de gérer ces agents. De simple fonctionnalités de gestions sont offertes par cette console.

\begin{itemize}
\item Stopper un agent
\item Lister les agents indépendamment de leur état
\begin{itemize}
\item Les agents en cours d'exécution
\item Les agents en pause
\item Les agents supprimer
\item Les agents en cours de test
\end{itemize}
\item Visualiser les statistiques d'un agent
\end{itemize}

Le fonctionnement de la console est assez simple. Par exemple, lorsque la commande d'arrêt est executée, un acteur est déployé sur le cluster afin d'envoyer le message d'arrêt à l'acteur désigné. Une fois le message délivré, l'acteur est stoppé. Cette console peut se connecter au cluster Akka et au cluster Kafka en fonction du type de commande exécutée.

### Visualiser les acteurs via un frontend {-}
Dans un premier temps, le frontend permettra seulement de visualiser les statistiques de l'ensemble des agents se trouvant sur le cluster. Le frontend recevra les données via un WebSocket connecté à un serveur node. Ce serveur aura pour rôle de consommer les données statistiques publiées dans le topic `statistics` du cluster Kafka. Il triera et mettra à jour l'historique de données statistiques de chaque acteurs en fonction de leur état.

\cimg{figs/create/create-node.png}{scale=0.5}{Architecture de déploiement du serveur node.js}{Source : Réalisé par Paschoud Nicolas}

\newpage

L'interface de visualisation des agents est assez simple. Ils sont présentés sous forme de liste de cartes contenant l'ensemble des informations d'un agent.

\cimg{figs/create/frontend-global.png}{scale=0.25}{Présentation du frontend}{Source : Réalisé par Paschoud Nicolas}

Un système de filtrage permet d'afficher les agents selon leur état. Il est aussi possible de chercher les agents à l'aide de leur nom.

\cimg{figs/create/front-search.png}{scale=0.4}{Menu de recherche des acteurs sur le frontend}{Source : Réalisé par Paschoud Nicolas}

\newpage

Leur état est défini par la couleur de leur en-tête de carte. Rouge indique que l'agent est arrêté, bleu indique qu'il est en cours d'exécution et jaune indique qu'il est en cours d'entrainement.

\cimg{figs/create/frontend-colors.png}{scale=0.3}{Exemple de couleurs pour définir l'état d'un acteur}{Source : Réalisé par Paschoud Nicolas}

## Synthèse {-}
Les outils présentés sont facilement modifiable et personnalisables si l'utilisateur souhaite déployer le système sur sa propre infrastructure. Le serveur web n'a besoin que de l'addresse du cluster Kafka pour fonctionner correctement.

\pagebreak